# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.

import os
from django.core import serializers
from django.core.exceptions import PermissionDenied
from django.db.models import get_model
from django.http import HttpResponse
from smuggler.settings import (SMUGGLER_EXCLUDE_LIST, SMUGGLER_FORMAT,
                               SMUGGLER_INDENT)

def get_excluded_models_set():
    excluded_models = set([])
    for label in SMUGGLER_EXCLUDE_LIST:
        app_label, model_label = label.split('.')
        excluded_models.add(get_model(app_label, model_label))
    return excluded_models

def get_file_list(path):
    file_list = []
    for file_name in os.listdir(path):
        if not os.path.isdir(file_name):
            file_path = os.path.join(path, file_name)
            file_size = os.path.getsize(file_path)
            file_list.append((file_name, '%0.1f KB'%float(file_size/1024.0)))
    file_list.sort()
    return file_list

def save_uploaded_file_on_disk(uploaded_file, destination_path):
    destination = open(destination_path, 'w')
    for chunk in uploaded_file.chunks():
        destination.write(chunk)
    destination.close()

def serialize_to_response(queryset, response=HttpResponse(),
                          format=SMUGGLER_FORMAT, indent=SMUGGLER_INDENT):
    serializers.serialize(format, queryset, indent=indent, stream=response)
    return response

def superuser_required(function):
    """
    Decorator for views that checks if the logged user is a superuser. In other
    words, deny access from non-superusers.
    """
    def _inner(request, *args, **kwargs):
        if not request.user.is_superuser:
            raise PermissionDenied
        return function(request, *args, **kwargs)
    return _inner



# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.

"""
Django Smuggler is a pluggable application for Django Web Framework that help
you dump/load fixtures via the automatically-generated admin interface.
"""

VERSION = (0, 2)

def get_version():
    """
    Returns the version as a human-format string.
    """
    return '.'.join([str(i) for i in VERSION])

__author__ = 'See the file AUTHORS.'
__license__ = 'GNU Lesser General Public License (GPL), Version 3'
__url__ = 'http://github.com/semente/django-smuggler'
__version__ = get_version()



# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.

import os
from datetime import datetime
from django.core import serializers
from django.db.models import get_app, get_apps, get_model, get_models
from django.http import HttpResponse, HttpResponseRedirect
from django.shortcuts import render_to_response
from django.template import RequestContext
from django.utils.translation import ugettext as _
from django.contrib import messages

from smuggler.forms import ImportFileForm
from smuggler.settings import SMUGGLER_FORMAT, SMUGGLER_FIXTURE_DIR
from smuggler.utils import (get_excluded_models_set, get_file_list,
                            save_uploaded_file_on_disk, serialize_to_response,
                            superuser_required)

def dump_data(request):
    """Exports data from whole project.
    """
    objects = []
    for app in get_apps():
        models = set(get_models(app)) - get_excluded_models_set()
        for model in models:
            if not model._meta.proxy:
                objects.extend(model._default_manager.all())
    filename = '%s.%s' % (datetime.now().isoformat(), SMUGGLER_FORMAT)
    response = HttpResponse(mimetype="text/plain")
    response['Content-Disposition'] = 'attachment; filename=%s' % filename
    return serialize_to_response(objects, response)
dump_data = superuser_required(dump_data)

def dump_app_data(request, app_label):
    """Exports data from a application.
    """
    objects = []
    models = set(get_models(get_app(app_label))) - get_excluded_models_set()
    for model in models:
        if not model._meta.proxy:
            objects.extend(model._default_manager.all())
    filename = '%s_%s.%s' % (app_label, datetime.now().isoformat(),
                             SMUGGLER_FORMAT)
    response = HttpResponse(mimetype="text/plain")
    response['Content-Disposition'] = 'attachment; filename=%s' % filename
    return serialize_to_response(objects, response)
dump_app_data = superuser_required(dump_app_data)

def dump_model_data(request, app_label, model_label):
    """Exports data from a model.
    """
    model = get_model(app_label, model_label)
    objects = model._default_manager.all()
    filename = '%s-%s_%s.%s' % (app_label, model_label,
                                datetime.now().isoformat(), SMUGGLER_FORMAT)
    response = HttpResponse(mimetype="text/plain")
    response['Content-Disposition'] = 'attachment; filename=%s' % filename
    return serialize_to_response(objects, response)
dump_model_data = superuser_required(dump_model_data)

def load_data(request):
    """
    Load data from uploaded file or disk.

    Note: A uploaded file will be saved on `SMUGGLER_FIXTURE_DIR` if the submit
          button with name "_loadandsave" was pressed.
    """
    form = ImportFileForm()
    if request.method == 'POST':
        data = []
        if request.POST.has_key('_load') or request.POST.has_key('_loadandsave'):
            form = ImportFileForm(request.POST, request.FILES)
            if form.is_valid():
                uploaded_file = request.FILES['file']
                file_name = uploaded_file.name
                file_format = file_name.split('.')[-1]
                if request.POST.has_key('_loadandsave'):
                    destination_path = os.path.join(SMUGGLER_FIXTURE_DIR,
                                                    file_name)
                    save_uploaded_file_on_disk(uploaded_file, destination_path)
                    file_data = open(destination_path, 'r')
                elif uploaded_file.multiple_chunks():
                    file_data = open(uploaded_file.temporary_file_path(), 'r')
                else:
                    file_data = uploaded_file.read()
                data.append((file_format, file_data))
        elif request.POST.has_key('_loadfromdisk'):
            query_dict = request.POST.copy()        
            del(query_dict['_loadfromdisk'])
            del(query_dict['csrfmiddlewaretoken'])
            selected_files = query_dict.values()
            for file_name in selected_files:
                file_path = os.path.join(SMUGGLER_FIXTURE_DIR, file_name)
                file_format = file_name.split('.')[-1]
                file_data = open(file_path, 'r')
                data.append((file_format, file_data))
        counter = 0
        for format, stream in data:
            objects = serializers.deserialize(format, stream)
            for obj in objects:
                counter += 1
                obj.save()
        if data:
            user_msg = ('%(obj_count)d object(s) from %(file_count)d file(s) '
                        'loaded with success.') # TODO: pluralize
            user_msg = _(user_msg) % {
                'obj_count': counter,
                'file_count': len(data)
            }
            messages.add_message(request, messages.INFO, user_msg)
    context = {
        'files_available': get_file_list(SMUGGLER_FIXTURE_DIR) \
            if SMUGGLER_FIXTURE_DIR else [],
        'smuggler_fixture_dir': SMUGGLER_FIXTURE_DIR,
        'import_file_form': form,
    }
    return render_to_response('smuggler/load_data_form.html', context,
                              context_instance=RequestContext(request))
load_data = superuser_required(load_data)



# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.

from django.conf.urls.defaults import *
from smuggler.views import (dump_data, dump_app_data, dump_model_data,
                            load_data)

dump_data_url = url(
    regex=r'^dump/$',
    view=dump_data,
    name='dump-data'
)

dump_app_data_url = url(
    regex=r'^(?P<app_label>\w+)/dump/$',
    view=dump_app_data,
    name='dump-app-data'
)

dump_model_data_url = url(
    regex=r'^(?P<app_label>\w+)/(?P<model_label>\w+)/dump/$',
    view=dump_model_data,
    name='dump-model-data'
)

load_data_url = url(
    regex=r'^load/$',
    view=load_data,
    name='load-data'
)

urlpatterns = patterns('', dump_model_data_url, dump_app_data_url,
                       dump_data_url, load_data_url)



# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.

from django import forms
from django.core.files.uploadedfile import InMemoryUploadedFile
from django.utils.translation import ugettext as _

class ImportFileForm(forms.Form):
    file = forms.FileField(
        label='File to load',
        help_text=_('Existing items with same <i>id</i> will be overwritten.'),
        required=True,
    )

    def clean_file(self):
        data = self.cleaned_data['file']
        if not isinstance(data, InMemoryUploadedFile):
            return data
        file_format = data.name.split('.')[-1]
        if not file_format in ['json', 'xml']:
            raise forms.ValidationError(_('Invalid file extension.'))
        return data



# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.

from datetime import datetime
from django.core import serializers
from django.core.exceptions import ImproperlyConfigured
from smuggler.settings import (SMUGGLER_FIXTURE_DIR, SMUGGLER_FORMAT,
                               SMUGGLER_INDENT)

def save_data_on_filesystem(sender, **kwargs):
    if not SMUGGLER_FIXTURE_DIR:
        raise ImproperlyConfigured('You need to specify SMUGGLER_FIXTURE_DIR in '
                                   'your Django settings file.')
    objects = sender._default_manager.all()
    app_label, model_label = sender._meta.app_label, sender._meta.module_name
    filename = '%s-%s_%s.%s' % (app_label, model_label,
                                datetime.now().isoformat(), SMUGGLER_FORMAT)
    fixture = file(SMUGGLER_FIXTURE_DIR + '/' + filename, 'w')
    serializers.serialize(SMUGGLER_FORMAT, objects, indent=SMUGGLER_INDENT,
                          stream=fixture)
    fixture.close()



# Copyright (c) 2009 Guilherme Gondim and contributors
#
# This file is part of Django Smuggler.
#
# Django Smuggler is free software under terms of the GNU Lesser
# General Public License version 3 (LGPLv3) as published by the Free
# Software Foundation. See the file README for copying conditions.

try:
    from setuptools import setup, find_packages
except ImportError:
    from ez_setup import use_setuptools
    use_setuptools()
    from setuptools import setup, find_packages

from smuggler import get_version

setup(
    name = 'django-smuggler',
    version = get_version(),
    description = 'dump/load fixtures via Django admin interface',
    long_description = ('Django Smuggler is a pluggable application for Django '
                        'Web Framework for you easily dump/load fixtures '
                        'via the automatically-generated administration '
                        'interface. Especially useful for transporting data in '
                        'production for the development project and vice'
                        'versa, but can also be used as a backup tool.'),
    keywords = 'django apps tools backup fixtures admin',
    author = 'Guilherme Gondim',
    author_email = 'semente@taurinus.org',
    url = 'http://github.com/semente/django-smuggler',
    download_url = 'http://github.com/semente/django-smuggler/downloads',
    license = 'GNU Lesser General Public License (LGPL), Version 3',
    classifiers = [
        'Environment :: Plugins',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
    packages = find_packages(),
    include_package_data = True,
    zip_safe = False,
)
